@@ -1,6 +1,4 @@
-// compile-pass
-
-#![feature(nll)]
+// build-pass (FIXME(62277): could be check-pass?)
 
 use std::ops::Deref;
 
