@@ -10,7 +10,7 @@ use std::io::{self, SeekFrom};
 use std::mem;
 use std::path::{Path, PathBuf};
 use std::str::FromStr;
-use std::sync::{Once, ONCE_INIT};
+use std::sync::Once;
 use std::time::Instant;
 use std::vec;
 
@@ -30,7 +30,7 @@ use crate::util::toml as cargo_toml;
 use crate::util::Filesystem;
 use crate::util::Rustc;
 use crate::util::{paths, validate_package_name, FileLock};
-use crate::util::{ToUrl, ToUrlWithBase};
+use crate::util::{IntoUrl, IntoUrlWithBase};
 
 /// Configuration information for cargo. This is not specific to a build, it is information
 /// relating to cargo itself.
@@ -83,13 +83,13 @@ pub struct Config {
     updated_sources: LazyCell<RefCell<HashSet<SourceId>>>,
     /// Lock, if held, of the global package cache along with the number of
     /// acquisitions so far.
-    package_cache_lock: RefCell<Option<(FileLock, usize)>>,
+    package_cache_lock: RefCell<Option<(Option<FileLock>, usize)>>,
 }
 
 impl Config {
     pub fn new(shell: Shell, cwd: PathBuf, homedir: PathBuf) -> Config {
         static mut GLOBAL_JOBSERVER: *mut jobserver::Client = 0 as *mut _;
-        static INIT: Once = ONCE_INIT;
+        static INIT: Once = Once::new();
 
         // This should be called early on in the process, so in theory the
         // unsafety is ok here. (taken ownership of random fds)
@@ -294,6 +294,13 @@ impl Config {
         self.values.try_borrow_with(|| self.load_values())
     }
 
+    pub fn values_mut(&mut self) -> CargoResult<&mut HashMap<String, ConfigValue>> {
+        match self.values.borrow_mut() {
+            Some(map) => Ok(map),
+            None => failure::bail!("config values not loaded yet"),
+        }
+    }
+
     // Note: this is used by RLS, not Cargo.
     pub fn set_values(&self, values: HashMap<String, ConfigValue>) -> CargoResult<()> {
         if self.values.borrow().is_some() {
@@ -704,11 +711,11 @@ impl Config {
     fn resolve_registry_index(&self, index: Value<String>) -> CargoResult<Url> {
         let base = index
             .definition
-            .root(&self)
+            .root(self)
             .join("truncated-by-url_with_base");
         // Parse val to check it is a URL, not a relative path without a protocol.
-        let _parsed = index.val.to_url()?;
-        let url = index.val.to_url_with_base(Some(&*base))?;
+        let _parsed = index.val.into_url()?;
+        let url = index.val.into_url_with_base(Some(&*base))?;
         if url.password().is_some() {
             failure::bail!("Registry URLs may not contain passwords");
         }
@@ -853,11 +860,11 @@ impl Config {
         let ret = f.as_path_unlocked();
         assert!(
             self.package_cache_lock.borrow().is_some(),
-            "pacakge cache lock is not currently held, Cargo forgot to call \
+            "package cache lock is not currently held, Cargo forgot to call \
              `acquire_package_cache_lock` before we got to this stack frame",
         );
         assert!(ret.starts_with(self.home_path.as_path_unlocked()));
-        return ret;
+        ret
     }
 
     /// Acquires an exclusive lock on the global "package cache"
@@ -865,7 +872,7 @@ impl Config {
     /// This lock is global per-process and can be acquired recursively. An RAII
     /// structure is returned to release the lock, and if this process
     /// abnormally terminates the lock is also released.
-    pub fn acquire_package_cache_lock<'a>(&'a self) -> CargoResult<PackageCacheLock<'a>> {
+    pub fn acquire_package_cache_lock(&self) -> CargoResult<PackageCacheLock<'_>> {
         let mut slot = self.package_cache_lock.borrow_mut();
         match *slot {
             // We've already acquired the lock in this process, so simply bump
@@ -875,33 +882,31 @@ impl Config {
             }
             None => {
                 let path = ".package-cache";
-                let desc = "package cache lock";
+                let desc = "package cache";
 
                 // First, attempt to open an exclusive lock which is in general
                 // the purpose of this lock!
                 //
-                // If that fails because of a readonly filesystem, though, then
-                // we don't want to fail because it's a readonly filesystem. In
-                // some situations Cargo is prepared to have a readonly
-                // filesystem yet still work since it's all been pre-downloaded
-                // and/or pre-unpacked. In these situations we want to keep
-                // Cargo running if possible, so if it's a readonly filesystem
-                // switch to a shared lock which should hopefully succeed so we
-                // can continue.
+                // If that fails because of a readonly filesystem or a
+                // permission error, though, then we don't really want to fail
+                // just because of this. All files that this lock protects are
+                // in subfolders, so they're assumed by Cargo to also be
+                // readonly or have invalid permissions for us to write to. If
+                // that's the case, then we don't really need to grab a lock in
+                // the first place here.
                 //
-                // Note that the package cache lock protects files in the same
-                // directory, so if it's a readonly filesystem we assume that
-                // the entire package cache is readonly, so we're just acquiring
-                // something to prove it works, we're not actually doing any
-                // synchronization at that point.
+                // Despite this we attempt to grab a readonly lock. This means
+                // that if our read-only folder is shared read-write with
+                // someone else on the system we should synchronize with them,
+                // but if we can't even do that then we did our best and we just
+                // keep on chugging elsewhere.
                 match self.home_path.open_rw(path, self, desc) {
-                    Ok(lock) => *slot = Some((lock, 1)),
+                    Ok(lock) => *slot = Some((Some(lock), 1)),
                     Err(e) => {
                         if maybe_readonly(&e) {
-                            if let Ok(lock) = self.home_path.open_ro(path, self, desc) {
-                                *slot = Some((lock, 1));
-                                return Ok(PackageCacheLock(self));
-                            }
+                            let lock = self.home_path.open_ro(path, self, desc).ok();
+                            *slot = Some((lock, 1));
+                            return Ok(PackageCacheLock(self));
                         }
 
                         Err(e).chain_err(|| "failed to acquire package cache lock")?;
@@ -1777,3 +1782,12 @@ impl Drop for PackageCacheLock<'_> {
         }
     }
 }
+
+/// returns path to clippy-driver binary
+///
+/// Allows override of the path via `CARGO_CLIPPY_DRIVER` env variable
+pub fn clippy_driver() -> PathBuf {
+    env::var("CARGO_CLIPPY_DRIVER")
+        .unwrap_or_else(|_| "clippy-driver".into())
+        .into()
+}
