@@ -1,10 +1,10 @@
 use if_chain::if_chain;
 use rustc::hir::def::{DefKind, Res};
+use rustc::hir::ptr::P;
 use rustc::hir::*;
 use rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
 use rustc::{declare_lint_pass, declare_tool_lint};
 use rustc_errors::Applicability;
-use syntax::ptr::P;
 
 use crate::utils::paths::*;
 use crate::utils::sugg::Sugg;
@@ -129,7 +129,7 @@ impl QuestionMark {
         }
     }
 
-    fn return_expression(block: &Block) -> Option<P<Expr>> {
+    fn return_expression(block: &Block) -> Option<&P<Expr>> {
         // Check if last expression is a return statement. Then, return the expression
         if_chain! {
             if block.stmts.len() == 1;
@@ -139,7 +139,7 @@ impl QuestionMark {
             if let &Some(ref ret_expr) = ret_expr;
 
             then {
-                return Some(ret_expr.clone());
+                return Some(ret_expr);
             }
         }
 
@@ -148,7 +148,7 @@ impl QuestionMark {
             if block.stmts.len() == 0;
             if let Some(ExprKind::Ret(Some(ret_expr))) = block.expr.as_ref().map(|e| &e.node);
             then {
-                return Some(ret_expr.clone());
+                return Some(ret_expr);
             }
         }
 
