@@ -2,8 +2,7 @@
 // compile-flags: -Zmiri-disable-validation
 
 #![feature(never_type)]
-#![allow(unreachable_code)]
-#![allow(unused_variables)]
+#![allow(unused)]
 
 enum Void {}
 
