@@ -1,49 +1,102 @@
-// Copyright 2015 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution and at
-// http://rust-lang.org/COPYRIGHT.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
-extern crate assert_cli;
-
-use std::collections::{HashMap, HashSet};
+use std::collections::HashMap;
 use std::env;
 use std::fs;
 use std::io::{self, BufRead, BufReader, Read, Write};
-use std::iter::{Enumerate, Peekable};
+use std::iter::Peekable;
 use std::mem;
 use std::path::{Path, PathBuf};
 use std::process::{Command, Stdio};
 use std::str::Chars;
+use std::thread;
 
-use config::{Color, Config, EmitMode, FileName, ReportTactic};
-use formatting::{ModifiedChunk, SourceFile};
-use rustfmt_diff::{make_diff, print_diff, DiffLine, Mismatch, OutputWriter};
-use source_file;
-use {FormatReport, Input, Session};
+use crate::config::{Color, Config, EmitMode, FileName, NewlineStyle, ReportTactic};
+use crate::formatting::{ReportedErrors, SourceFile};
+use crate::is_nightly_channel;
+use crate::rustfmt_diff::{make_diff, print_diff, DiffLine, Mismatch, ModifiedChunk, OutputWriter};
+use crate::source_file;
+use crate::{FormatReport, FormatReportFormatterBuilder, Input, Session};
+
+mod configuration_snippet;
 
 const DIFF_CONTEXT_SIZE: usize = 3;
-const CONFIGURATIONS_FILE_NAME: &str = "Configurations.md";
 
-// Returns a `Vec` containing `PathBuf`s of files with a rs extension in the
+// A list of files on which we want to skip testing.
+const SKIP_FILE_WHITE_LIST: &[&str] = &[
+    // We want to make sure that the `skip_children` is correctly working,
+    // so we do not want to test this file directly.
+    "configs/skip_children/foo/mod.rs",
+    "issue-3434/no_entry.rs",
+    "issue-3665/sub_mod.rs",
+    // These files and directory are a part of modules defined inside `cfg_if!`.
+    "cfg_if/mod.rs",
+    "cfg_if/detect",
+    // These files and directory are a part of modules defined inside `cfg_attr(..)`.
+    "cfg_mod/dir",
+    "cfg_mod/bar.rs",
+    "cfg_mod/foo.rs",
+    "cfg_mod/wasm32.rs",
+];
+
+struct TestSetting {
+    /// The size of the stack of the thread that run tests.
+    stack_size: usize,
+}
+
+impl Default for TestSetting {
+    fn default() -> Self {
+        TestSetting {
+            stack_size: 8_388_608, // 8MB
+        }
+    }
+}
+
+fn run_test_with<F>(test_setting: &TestSetting, f: F)
+where
+    F: FnOnce(),
+    F: Send + 'static,
+{
+    thread::Builder::new()
+        .stack_size(test_setting.stack_size)
+        .spawn(f)
+        .expect("Failed to create a test thread")
+        .join()
+        .expect("Failed to join a test thread")
+}
+
+fn is_subpath<P>(path: &Path, subpath: &P) -> bool
+where
+    P: AsRef<Path>,
+{
+    (0..path.components().count())
+        .map(|i| {
+            path.components()
+                .skip(i)
+                .take(subpath.as_ref().components().count())
+        })
+        .any(|c| c.zip(subpath.as_ref().components()).all(|(a, b)| a == b))
+}
+
+fn is_file_skip(path: &Path) -> bool {
+    SKIP_FILE_WHITE_LIST
+        .iter()
+        .any(|file_path| is_subpath(path, file_path))
+}
+
+// Returns a `Vec` containing `PathBuf`s of files with an  `rs` extension in the
 // given path. The `recursive` argument controls if files from subdirectories
 // are also returned.
 fn get_test_files(path: &Path, recursive: bool) -> Vec<PathBuf> {
     let mut files = vec![];
     if path.is_dir() {
         for entry in fs::read_dir(path).expect(&format!(
-            "Couldn't read directory {}",
+            "couldn't read directory {}",
             path.to_str().unwrap()
         )) {
-            let entry = entry.expect("Couldn't get DirEntry");
+            let entry = entry.expect("couldn't get `DirEntry`");
             let path = entry.path();
             if path.is_dir() && recursive {
                 files.append(&mut get_test_files(&path, recursive));
-            } else if path.extension().map_or(false, |f| f == "rs") {
+            } else if path.extension().map_or(false, |f| f == "rs") && !is_file_skip(&path) {
                 files.push(path);
             }
         }
@@ -53,20 +106,21 @@ fn get_test_files(path: &Path, recursive: bool) -> Vec<PathBuf> {
 
 fn verify_config_used(path: &Path, config_name: &str) {
     for entry in fs::read_dir(path).expect(&format!(
-        "Couldn't read {} directory",
+        "couldn't read {} directory",
         path.to_str().unwrap()
     )) {
-        let entry = entry.expect("Couldn't get directory entry");
+        let entry = entry.expect("couldn't get directory entry");
         let path = entry.path();
         if path.extension().map_or(false, |f| f == "rs") {
             // check if "// rustfmt-<config_name>:" appears in the file.
             let filebuf = BufReader::new(
-                fs::File::open(&path).expect(&format!("Couldn't read file {}", path.display())),
+                fs::File::open(&path)
+                    .unwrap_or_else(|_| panic!("couldn't read file {}", path.display())),
             );
             assert!(
                 filebuf
                     .lines()
-                    .map(|l| l.unwrap())
+                    .map(Result::unwrap)
                     .take_while(|l| l.starts_with("//"))
                     .any(|l| l.starts_with(&format!("// rustfmt-{}", config_name))),
                 format!(
@@ -84,8 +138,8 @@ fn verify_config_test_names() {
         Path::new("tests/source/configs"),
         Path::new("tests/target/configs"),
     ] {
-        for entry in fs::read_dir(path).expect("Couldn't read configs directory") {
-            let entry = entry.expect("Couldn't get directory entry");
+        for entry in fs::read_dir(path).expect("couldn't read configs directory") {
+            let entry = entry.expect("couldn't get directory entry");
             let path = entry.path();
             if path.is_dir() {
                 let config_name = path.file_name().unwrap().to_str().unwrap();
@@ -97,8 +151,8 @@ fn verify_config_test_names() {
     }
 }
 
-// This writes to the terminal using the same approach (via term::stdout or
-// println!) that is used by `rustfmt::rustfmt_diff::print_diff`. Writing
+// This writes to the terminal using the same approach (via `term::stdout` or
+// `println!`) that is used by `rustfmt::rustfmt_diff::print_diff`. Writing
 // using only one or the other will cause the output order to differ when
 // `print_diff` selects the approach not used.
 fn write_message(msg: &str) {
@@ -106,23 +160,30 @@ fn write_message(msg: &str) {
     writer.writeln(msg, None);
 }
 
-// Integration tests. The files in the tests/source are formatted and compared
-// to their equivalent in tests/target. The target file and config can be
+// Integration tests. The files in `tests/source` are formatted and compared
+// to their equivalent in `tests/target`. The target file and config can be
 // overridden by annotations in the source file. The input and output must match
 // exactly.
 #[test]
 fn system_tests() {
-    // Get all files in the tests/source directory.
-    let files = get_test_files(Path::new("tests/source"), true);
-    let (_reports, count, fails) = check_files(files, &None);
+    run_test_with(&TestSetting::default(), || {
+        // Get all files in the tests/source directory.
+        let files = get_test_files(Path::new("tests/source"), true);
+        let (_reports, count, fails) = check_files(files, &None);
 
-    // Display results.
-    println!("Ran {} system tests.", count);
-    assert_eq!(fails, 0, "{} system tests failed", fails);
+        // Display results.
+        println!("Ran {} system tests.", count);
+        assert_eq!(fails, 0, "{} system tests failed", fails);
+        assert!(
+            count >= 300,
+            "Expected a minimum of {} system tests to be executed",
+            300
+        )
+    });
 }
 
-// Do the same for tests/coverage-source directory
-// the only difference is the coverage mode
+// Do the same for tests/coverage-source directory.
+// The only difference is the coverage mode.
 #[test]
 fn coverage_tests() {
     let files = get_test_files(Path::new("tests/coverage/source"), true);
@@ -147,7 +208,9 @@ fn modified_test() {
     let filename = "tests/writemode/source/modified.rs";
     let mut data = Vec::new();
     let mut config = Config::default();
-    config.set().emit_mode(::config::EmitMode::ModifiedLines);
+    config
+        .set()
+        .emit_mode(crate::config::EmitMode::ModifiedLines);
 
     {
         let mut session = Session::new(config, Some(&mut data));
@@ -205,7 +268,7 @@ fn assert_output(source: &Path, expected_filename: &Path) {
     let _ = source_file::write_all_files(&source_file, &mut out, &config);
     let output = String::from_utf8(out).unwrap();
 
-    let mut expected_file = fs::File::open(&expected_filename).expect("Couldn't open target");
+    let mut expected_file = fs::File::open(&expected_filename).expect("couldn't open target");
     let mut expected_text = String::new();
     expected_file
         .read_to_string(&mut expected_text)
@@ -216,7 +279,7 @@ fn assert_output(source: &Path, expected_filename: &Path) {
         let mut failures = HashMap::new();
         failures.insert(source.to_owned(), compare);
         print_mismatches_default_message(failures);
-        assert!(false, "Text does not match expected output");
+        panic!("Text does not match expected output");
     }
 }
 
@@ -224,23 +287,34 @@ fn assert_output(source: &Path, expected_filename: &Path) {
 // rustfmt.
 #[test]
 fn idempotence_tests() {
-    match option_env!("CFG_RELEASE_CHANNEL") {
-        None | Some("nightly") => {}
-        _ => return, // these tests require nightly
-    }
-    // Get all files in the tests/target directory.
-    let files = get_test_files(Path::new("tests/target"), true);
-    let (_reports, count, fails) = check_files(files, &None);
+    run_test_with(&TestSetting::default(), || {
+        // these tests require nightly
+        if !is_nightly_channel!() {
+            return;
+        }
+        // Get all files in the tests/target directory.
+        let files = get_test_files(Path::new("tests/target"), true);
+        let (_reports, count, fails) = check_files(files, &None);
 
-    // Display results.
-    println!("Ran {} idempotent tests.", count);
-    assert_eq!(fails, 0, "{} idempotent tests failed", fails);
+        // Display results.
+        println!("Ran {} idempotent tests.", count);
+        assert_eq!(fails, 0, "{} idempotent tests failed", fails);
+        assert!(
+            count >= 400,
+            "Expected a minimum of {} idempotent tests to be executed",
+            400
+        )
+    });
 }
 
 // Run rustfmt on itself. This operation must be idempotent. We also check that
 // no warnings are emitted.
 #[test]
 fn self_tests() {
+    // Issue-3443: these tests require nightly
+    if !is_nightly_channel!() {
+        return;
+    }
     let mut files = get_test_files(Path::new("tests"), false);
     let bin_directories = vec!["cargo-fmt", "git-rustfmt", "bin", "format-diff"];
     for dir in bin_directories {
@@ -259,7 +333,10 @@ fn self_tests() {
     assert_eq!(fails, 0, "{} self tests failed", fails);
 
     for format_report in reports {
-        println!("{}", format_report);
+        println!(
+            "{}",
+            FormatReportFormatterBuilder::new(&format_report).build()
+        );
         warnings += format_report.warning_count();
     }
 
@@ -283,19 +360,56 @@ fn stdin_formatting_smoke_test() {
     }
 
     #[cfg(not(windows))]
-    assert_eq!(buf, "fn main() {}\n".as_bytes());
+    assert_eq!(buf, "stdin:\n\nfn main() {}\n".as_bytes());
     #[cfg(windows)]
-    assert_eq!(buf, "fn main() {}\r\n".as_bytes());
+    assert_eq!(buf, "stdin:\n\nfn main() {}\r\n".as_bytes());
+}
+
+#[test]
+fn stdin_parser_panic_caught() {
+    // See issue #3239.
+    for text in ["{", "}"].iter().cloned().map(String::from) {
+        let mut buf = vec![];
+        let mut session = Session::new(Default::default(), Some(&mut buf));
+        let _ = session.format(Input::Text(text));
+
+        assert!(session.has_parsing_errors());
+    }
+}
+
+/// Ensures that `EmitMode::ModifiedLines` works with input from `stdin`. Useful
+/// when embedding Rustfmt (e.g. inside RLS).
+#[test]
+fn stdin_works_with_modified_lines() {
+    let input = "\nfn\n some( )\n{\n}\nfn main () {}\n";
+    let output = "1 6 2\nfn some() {}\nfn main() {}\n";
+
+    let input = Input::Text(input.to_owned());
+    let mut config = Config::default();
+    config.set().newline_style(NewlineStyle::Unix);
+    config.set().emit_mode(EmitMode::ModifiedLines);
+    let mut buf: Vec<u8> = vec![];
+    {
+        let mut session = Session::new(config, Some(&mut buf));
+        session.format(input).unwrap();
+        let errors = ReportedErrors {
+            has_diff: true,
+            ..Default::default()
+        };
+        assert_eq!(session.errors, errors);
+    }
+    assert_eq!(buf, output.as_bytes());
 }
 
 #[test]
 fn stdin_disable_all_formatting_test() {
     match option_env!("CFG_RELEASE_CHANNEL") {
         None | Some("nightly") => {}
-        _ => return, // these tests require nightly
+        // These tests require nightly.
+        _ => return,
     }
     let input = String::from("fn main() { println!(\"This should not be formatted.\"); }");
-    let mut child = Command::new("./target/debug/rustfmt")
+    let mut child = Command::new(rustfmt().to_str().unwrap())
         .stdin(Stdio::piped())
         .stdout(Stdio::piped())
         .arg("--config-path=./tests/config/disable_all_formatting.toml")
@@ -346,11 +460,21 @@ fn check_files(files: Vec<PathBuf>, opt_config: &Option<PathBuf>) -> (Vec<Format
     let mut reports = vec![];
 
     for file_name in files {
+        let sig_comments = read_significant_comments(&file_name);
+        if sig_comments.contains_key("unstable") && !is_nightly_channel!() {
+            debug!(
+                "Skipping '{}' because it requires unstable \
+                 features which are only available on nightly...",
+                file_name.display()
+            );
+            continue;
+        }
+
         debug!("Testing '{}'...", file_name.display());
 
         match idempotent_check(&file_name, &opt_config) {
             Ok(ref report) if report.has_warnings() => {
-                print!("{}", report);
+                print!("{}", FormatReportFormatterBuilder::new(&report).build());
                 fails += 1;
             }
             Ok(report) => reports.push(report),
@@ -395,7 +519,7 @@ fn print_mismatches<T: Fn(u32) -> String>(
 
 fn read_config(filename: &Path) -> Config {
     let sig_comments = read_significant_comments(filename);
-    // Look for a config file... If there is a 'config' property in the significant comments, use
+    // Look for a config file. If there is a 'config' property in the significant comments, use
     // that. Otherwise, if there are no significant comments at all, look for a config file with
     // the same name as the test file.
     let mut config = if !sig_comments.is_empty() {
@@ -405,7 +529,7 @@ fn read_config(filename: &Path) -> Config {
     };
 
     for (key, val) in &sig_comments {
-        if key != "target" && key != "config" {
+        if key != "target" && key != "config" && key != "unstable" {
             config.override_value(key, val);
             if config.is_default(key) {
                 warn!("Default value {} used explicitly for {}", val, key);
@@ -441,7 +565,7 @@ fn idempotent_check(
 ) -> Result<FormatReport, IdempotentCheckError> {
     let sig_comments = read_significant_comments(filename);
     let config = if let Some(ref config_file_path) = opt_config {
-        Config::from_toml_path(config_file_path).expect("rustfmt.toml not found")
+        Config::from_toml_path(config_file_path).expect("`rustfmt.toml` not found")
     } else {
         read_config(filename)
     };
@@ -478,43 +602,42 @@ fn get_config(config_file: Option<&Path>) -> Config {
         }
     };
 
-    let mut def_config_file = fs::File::open(config_file_name).expect("Couldn't open config");
+    let mut def_config_file = fs::File::open(config_file_name).expect("couldn't open config");
     let mut def_config = String::new();
     def_config_file
         .read_to_string(&mut def_config)
         .expect("Couldn't read config");
 
-    Config::from_toml(&def_config, Path::new("tests/config/")).expect("Invalid toml")
+    Config::from_toml(&def_config, Path::new("tests/config/")).expect("invalid TOML")
 }
 
-// Reads significant comments of the form: // rustfmt-key: value
-// into a hash map.
+// Reads significant comments of the form: `// rustfmt-key: value` into a hash map.
 fn read_significant_comments(file_name: &Path) -> HashMap<String, String> {
-    let file =
-        fs::File::open(file_name).expect(&format!("Couldn't read file {}", file_name.display()));
+    let file = fs::File::open(file_name)
+        .unwrap_or_else(|_| panic!("couldn't read file {}", file_name.display()));
     let reader = BufReader::new(file);
     let pattern = r"^\s*//\s*rustfmt-([^:]+):\s*(\S+)";
-    let regex = regex::Regex::new(pattern).expect("Failed creating pattern 1");
+    let regex = regex::Regex::new(pattern).expect("failed creating pattern 1");
 
     // Matches lines containing significant comments or whitespace.
     let line_regex = regex::Regex::new(r"(^\s*$)|(^\s*//\s*rustfmt-[^:]+:\s*\S+)")
-        .expect("Failed creating pattern 2");
+        .expect("failed creating pattern 2");
 
     reader
         .lines()
-        .map(|line| line.expect("Failed getting line"))
+        .map(|line| line.expect("failed getting line"))
         .take_while(|line| line_regex.is_match(line))
         .filter_map(|line| {
             regex.captures_iter(&line).next().map(|capture| {
                 (
                     capture
                         .get(1)
-                        .expect("Couldn't unwrap capture")
+                        .expect("couldn't unwrap capture")
                         .as_str()
                         .to_owned(),
                     capture
                         .get(2)
-                        .expect("Couldn't unwrap capture")
+                        .expect("couldn't unwrap capture")
                         .as_str()
                         .to_owned(),
                 )
@@ -523,7 +646,7 @@ fn read_significant_comments(file_name: &Path) -> HashMap<String, String> {
         .collect()
 }
 
-// Compare output to input.
+// Compares output to input.
 // TODO: needs a better name, more explanation.
 fn handle_result(
     result: HashMap<PathBuf, String>,
@@ -534,11 +657,11 @@ fn handle_result(
     for (file_name, fmt_text) in result {
         // If file is in tests/source, compare to file with same name in tests/target.
         let target = get_target(&file_name, target);
-        let open_error = format!("Couldn't open target {:?}", &target);
+        let open_error = format!("couldn't open target {:?}", target);
         let mut f = fs::File::open(&target).expect(&open_error);
 
         let mut text = String::new();
-        let read_error = format!("Failed reading target {:?}", &target);
+        let read_error = format!("failed reading target {:?}", target);
         f.read_to_string(&mut text).expect(&read_error);
 
         // Ignore LF and CRLF difference for Windows.
@@ -559,7 +682,7 @@ fn handle_result(
     }
 }
 
-// Map source file paths to their target paths.
+// Maps source file paths to their target paths.
 fn get_target(file_name: &Path, target: Option<&str>) -> PathBuf {
     if let Some(n) = file_name
         .components()
@@ -579,7 +702,7 @@ fn get_target(file_name: &Path, target: Option<&str>) -> PathBuf {
             target_file_name
         }
     } else {
-        // This is either and idempotence check or a self check
+        // This is either and idempotence check or a self check.
         file_name.to_owned()
     }
 }
@@ -647,280 +770,6 @@ fn string_eq_ignore_newline_repr_test() {
     assert!(!string_eq_ignore_newline_repr("a\r\nbcd", "a\nbcdefghijk"));
 }
 
-// This enum is used to represent one of three text features in Configurations.md: a block of code
-// with its starting line number, the name of a rustfmt configuration option, or the value of a
-// rustfmt configuration option.
-enum ConfigurationSection {
-    CodeBlock((String, u32)), // (String: block of code, u32: line number of code block start)
-    ConfigName(String),
-    ConfigValue(String),
-}
-
-impl ConfigurationSection {
-    fn get_section<I: Iterator<Item = String>>(
-        file: &mut Enumerate<I>,
-    ) -> Option<ConfigurationSection> {
-        lazy_static! {
-            static ref CONFIG_NAME_REGEX: regex::Regex =
-                regex::Regex::new(r"^## `([^`]+)`").expect("Failed creating configuration pattern");
-            static ref CONFIG_VALUE_REGEX: regex::Regex =
-                regex::Regex::new(r#"^#### `"?([^`"]+)"?`"#)
-                    .expect("Failed creating configuration value pattern");
-        }
-
-        loop {
-            match file.next() {
-                Some((i, line)) => {
-                    if line.starts_with("```rust") {
-                        // Get the lines of the code block.
-                        let lines: Vec<String> = file
-                            .map(|(_i, l)| l)
-                            .take_while(|l| !l.starts_with("```"))
-                            .collect();
-                        let block = format!("{}\n", lines.join("\n"));
-
-                        // +1 to translate to one-based indexing
-                        // +1 to get to first line of code (line after "```")
-                        let start_line = (i + 2) as u32;
-
-                        return Some(ConfigurationSection::CodeBlock((block, start_line)));
-                    } else if let Some(c) = CONFIG_NAME_REGEX.captures(&line) {
-                        return Some(ConfigurationSection::ConfigName(String::from(&c[1])));
-                    } else if let Some(c) = CONFIG_VALUE_REGEX.captures(&line) {
-                        return Some(ConfigurationSection::ConfigValue(String::from(&c[1])));
-                    }
-                }
-                None => return None, // reached the end of the file
-            }
-        }
-    }
-}
-
-// This struct stores the information about code blocks in the configurations
-// file, formats the code blocks, and prints formatting errors.
-struct ConfigCodeBlock {
-    config_name: Option<String>,
-    config_value: Option<String>,
-    code_block: Option<String>,
-    code_block_start: Option<u32>,
-}
-
-impl ConfigCodeBlock {
-    fn new() -> ConfigCodeBlock {
-        ConfigCodeBlock {
-            config_name: None,
-            config_value: None,
-            code_block: None,
-            code_block_start: None,
-        }
-    }
-
-    fn set_config_name(&mut self, name: Option<String>) {
-        self.config_name = name;
-        self.config_value = None;
-    }
-
-    fn set_config_value(&mut self, value: Option<String>) {
-        self.config_value = value;
-    }
-
-    fn set_code_block(&mut self, code_block: String, code_block_start: u32) {
-        self.code_block = Some(code_block);
-        self.code_block_start = Some(code_block_start);
-    }
-
-    fn get_block_config(&self) -> Config {
-        let mut config = Config::default();
-        if self.config_value.is_some() && self.config_value.is_some() {
-            config.override_value(
-                self.config_name.as_ref().unwrap(),
-                self.config_value.as_ref().unwrap(),
-            );
-        }
-        config
-    }
-
-    fn code_block_valid(&self) -> bool {
-        // We never expect to not have a code block.
-        assert!(self.code_block.is_some() && self.code_block_start.is_some());
-
-        // See if code block begins with #![rustfmt::skip].
-        let fmt_skip = self
-            .code_block
-            .as_ref()
-            .unwrap()
-            .lines()
-            .nth(0)
-            .unwrap_or("")
-            == "#![rustfmt::skip]";
-
-        if self.config_name.is_none() && !fmt_skip {
-            write_message(&format!(
-                "No configuration name for {}:{}",
-                CONFIGURATIONS_FILE_NAME,
-                self.code_block_start.unwrap()
-            ));
-            return false;
-        }
-        if self.config_value.is_none() && !fmt_skip {
-            write_message(&format!(
-                "No configuration value for {}:{}",
-                CONFIGURATIONS_FILE_NAME,
-                self.code_block_start.unwrap()
-            ));
-            return false;
-        }
-        true
-    }
-
-    fn has_parsing_errors<T: Write>(&self, session: &Session<T>) -> bool {
-        if session.has_parsing_errors() {
-            write_message(&format!(
-                "\u{261d}\u{1f3fd} Cannot format {}:{}",
-                CONFIGURATIONS_FILE_NAME,
-                self.code_block_start.unwrap()
-            ));
-            return true;
-        }
-
-        false
-    }
-
-    fn print_diff(&self, compare: Vec<Mismatch>) {
-        let mut mismatches = HashMap::new();
-        mismatches.insert(PathBuf::from(CONFIGURATIONS_FILE_NAME), compare);
-        print_mismatches(mismatches, |line_num| {
-            format!(
-                "\nMismatch at {}:{}:",
-                CONFIGURATIONS_FILE_NAME,
-                line_num + self.code_block_start.unwrap() - 1
-            )
-        });
-    }
-
-    fn formatted_has_diff(&self, text: &str) -> bool {
-        let compare = make_diff(self.code_block.as_ref().unwrap(), text, DIFF_CONTEXT_SIZE);
-        if !compare.is_empty() {
-            self.print_diff(compare);
-            return true;
-        }
-
-        false
-    }
-
-    // Return a bool indicating if formatting this code block is an idempotent
-    // operation. This function also triggers printing any formatting failure
-    // messages.
-    fn formatted_is_idempotent(&self) -> bool {
-        // Verify that we have all of the expected information.
-        if !self.code_block_valid() {
-            return false;
-        }
-
-        let input = Input::Text(self.code_block.as_ref().unwrap().to_owned());
-        let mut config = self.get_block_config();
-        config.set().emit_mode(EmitMode::Stdout);
-        let mut buf: Vec<u8> = vec![];
-
-        {
-            let mut session = Session::new(config, Some(&mut buf));
-            session.format(input).unwrap();
-            if self.has_parsing_errors(&session) {
-                return false;
-            }
-        }
-
-        !self.formatted_has_diff(&String::from_utf8(buf).unwrap())
-    }
-
-    // Extract a code block from the iterator. Behavior:
-    // - Rust code blocks are identifed by lines beginning with "```rust".
-    // - One explicit configuration setting is supported per code block.
-    // - Rust code blocks with no configuration setting are illegal and cause an
-    //   assertion failure, unless the snippet begins with #![rustfmt::skip].
-    // - Configuration names in Configurations.md must be in the form of
-    //   "## `NAME`".
-    // - Configuration values in Configurations.md must be in the form of
-    //   "#### `VALUE`".
-    fn extract<I: Iterator<Item = String>>(
-        file: &mut Enumerate<I>,
-        prev: Option<&ConfigCodeBlock>,
-        hash_set: &mut HashSet<String>,
-    ) -> Option<ConfigCodeBlock> {
-        let mut code_block = ConfigCodeBlock::new();
-        code_block.config_name = prev.and_then(|cb| cb.config_name.clone());
-
-        loop {
-            match ConfigurationSection::get_section(file) {
-                Some(ConfigurationSection::CodeBlock((block, start_line))) => {
-                    code_block.set_code_block(block, start_line);
-                    break;
-                }
-                Some(ConfigurationSection::ConfigName(name)) => {
-                    assert!(
-                        Config::is_valid_name(&name),
-                        "an unknown configuration option was found: {}",
-                        name
-                    );
-                    assert!(
-                        hash_set.remove(&name),
-                        "multiple configuration guides found for option {}",
-                        name
-                    );
-                    code_block.set_config_name(Some(name));
-                }
-                Some(ConfigurationSection::ConfigValue(value)) => {
-                    code_block.set_config_value(Some(value));
-                }
-                None => return None, // end of file was reached
-            }
-        }
-
-        Some(code_block)
-    }
-}
-
-#[test]
-fn configuration_snippet_tests() {
-    // Read Configurations.md and build a `Vec` of `ConfigCodeBlock` structs with one
-    // entry for each Rust code block found.
-    fn get_code_blocks() -> Vec<ConfigCodeBlock> {
-        let mut file_iter = BufReader::new(
-            fs::File::open(Path::new(CONFIGURATIONS_FILE_NAME))
-                .expect(&format!("Couldn't read file {}", CONFIGURATIONS_FILE_NAME)),
-        )
-        .lines()
-        .map(|l| l.unwrap())
-        .enumerate();
-        let mut code_blocks: Vec<ConfigCodeBlock> = Vec::new();
-        let mut hash_set = Config::hash_set();
-
-        while let Some(cb) =
-            ConfigCodeBlock::extract(&mut file_iter, code_blocks.last(), &mut hash_set)
-        {
-            code_blocks.push(cb);
-        }
-
-        for name in hash_set {
-            if !Config::is_hidden_option(&name) {
-                panic!("{} does not have a configuration guide", name);
-            }
-        }
-
-        code_blocks
-    }
-
-    let blocks = get_code_blocks();
-    let failures = blocks
-        .iter()
-        .map(|b| b.formatted_is_idempotent())
-        .fold(0, |acc, r| acc + (!r as u32));
-
-    // Display results.
-    println!("Ran {} configurations tests.", blocks.len());
-    assert_eq!(failures, 0, "{} configurations tests failed", failures);
-}
-
 struct TempFile {
     path: PathBuf,
 }
@@ -933,28 +782,30 @@ fn make_temp_file(file_name: &'static str) -> TempFile {
     let target_dir = var("RUSTFMT_TEST_DIR").unwrap_or_else(|_| ".".to_owned());
     let path = Path::new(&target_dir).join(file_name);
 
-    let mut file = File::create(&path).expect("Couldn't create temp file");
+    let mut file = File::create(&path).expect("couldn't create temp file");
     let content = "fn main() {}\n";
     file.write_all(content.as_bytes())
-        .expect("Couldn't write temp file");
+        .expect("couldn't write temp file");
     TempFile { path }
 }
 
 impl Drop for TempFile {
     fn drop(&mut self) {
         use std::fs::remove_file;
-        remove_file(&self.path).expect("Couldn't delete temp file");
+        remove_file(&self.path).expect("couldn't delete temp file");
     }
 }
 
 fn rustfmt() -> PathBuf {
     let mut me = env::current_exe().expect("failed to get current executable");
-    me.pop(); // chop of the test name
-    me.pop(); // chop off `deps`
+    // Chop of the test name.
+    me.pop();
+    // Chop off `deps`.
+    me.pop();
 
-    // if we run `cargo test --release` we might only have a release build
+    // If we run `cargo test --release`, we might only have a release build.
     if cfg!(release) {
-        // ../release/
+        // `../release/`
         me.pop();
         me.push("release");
     }
@@ -973,11 +824,10 @@ fn rustfmt() -> PathBuf {
 #[test]
 fn verify_check_works() {
     let temp_file = make_temp_file("temp_check.rs");
-    assert_cli::Assert::command(&[
-        rustfmt().to_str().unwrap(),
-        "--check",
-        temp_file.path.to_str().unwrap(),
-    ])
-    .succeeds()
-    .unwrap();
+
+    Command::new(rustfmt().to_str().unwrap())
+        .arg("--check")
+        .arg(temp_file.path.to_str().unwrap())
+        .status()
+        .expect("run with check option failed");
 }
