@@ -6,8 +6,6 @@ itemmacro!(really, long.aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
 
 itemmacro!{this, is.brace().formatted()}
 
-peg_file!   modname  ("mygrammarfile.rustpeg");
-
 fn main() {
     foo! ( );
 
@@ -474,7 +472,6 @@ pub fn fold_abi<V: Fold + ?Sized>(_visitor: &mut V, _i: Abi) -> Abi {
 
 // #3463
 x ! {()}
-x ! y {()}
 
 // #3583
 foo!(|x = y|);
